// Penrose tiling
//
// rewrote Python code in http://preshing.com/20110831/penrose-tiling-explained
// for Ruby using TurtleField
// Y. Hayakawa, 2013
// 22-11-2021 Golang

package main

import (
	"tfield/turtle"
	"math"
	"math/cmplx"
)

var (
	GOLDENRATIO = complex( (1 + math.Sqrt(5.0)) / 2, 0.0)
)

type Triangle struct {
	Color int
	P0 complex128
	P1 complex128
	P2 complex128
}

type PenroseTile struct {
	T turtle.Turtle
	Triangles *[]Triangle
	Ttl *turtle.Turtle
}

func (pt *PenroseTile) Subdivide(triangles []Triangle) []Triangle {
    result := make([]Triangle,0)
    for _, t := range triangles {
		a := t.P0
		b := t.P1
		c := t.P2	
		if t.Color == 0 {
			p := a + (b - a) / GOLDENRATIO
			result = append(result,Triangle{0, c, p, b})
			result = append(result,Triangle{1, p, c, a})
		} else {
			q := b + (a - b) / GOLDENRATIO
			r := b + (c - b) / GOLDENRATIO
			result = append(result,Triangle{1, r, c, a})
			result = append(result,Triangle{1, q, r, b})
			result = append(result,Triangle{0, r, q, a})
	  }
	}
    return result
}

func (pt *PenroseTile) Gen(level int) {
	triangles := make([]Triangle, 0)
    for i:=0;i<10;i++ {
		b := cmplx.Rect(1, float64(2*i - 1) * math.Pi / 10.0)
		c := cmplx.Rect(1, float64(2*i + 1) * math.Pi / 10.0)
		if i % 2 == 0 {
			b,c = c,b
		}
		triangles = append(triangles,Triangle{0, complex(0,0), b, c})
	}
    for i:=0;i<level;i++ {
		triangles = pt.Subdivide(triangles)
	}
	pt.Triangles = &triangles
	return
}

func (pt *PenroseTile) Draw(mag float64) {
	pt.Ttl.Clr()
	for _,t := range *pt.Triangles {
		pt.Ttl.Fill()
		if t.Color == 0 {
			pt.Ttl.Col(1.0,0.3,0.0)
		} else {
			pt.Ttl.Col(0.3,0.2,1.0)
		}

		a := t.P0
		b := t.P1
		c := t.P2
		
		pt.Ttl.Jump(real(a)*mag, imag(a)*mag) 
		pt.Ttl.Pd()
		pt.Ttl.Jump(real(b)*mag, imag(b)*mag) 
		pt.Ttl.Jump(real(c)*mag, imag(c)*mag) 
		pt.Ttl.Pu()

		pt.Ttl.Col(0.0,0.0,0.0)
		pt.Ttl.Line()

		pt.Ttl.Jump(real(b)*mag, imag(b)*mag) 
		pt.Ttl.Pd()
		pt.Ttl.Jump(real(a)*mag, imag(a)*mag) 
		pt.Ttl.Jump(real(c)*mag, imag(c)*mag) 
		pt.Ttl.Pu()
	}
}

func main() {
	ttl := new(turtle.Turtle)
	ttl.Connect("localhost")

	tile := new(PenroseTile)
	tile.Ttl = ttl

	tile.Gen(5)
	tile.Draw(250.0)
}
